import { v4 as uuid } from 'uuid';
/**
 * Remove a content id from a block's content array. Enter the block you want to unreference (not the parent block)
 * @param {Block[]} blocks - array of blocks
 * @param {Id} id - id of the block to remove
 * @returns {Block[]} updated blocks array
 */
export function removeRefToChild({ blocks, id }) {
    // console.log('delBlock', id)
    const copiedBlocks = [...blocks];
    const blockToUnlink = getBlock({ blocks, id });
    if (blockToUnlink.parent) {
        const parentBlock = getBlock({ blocks, id: blockToUnlink.parent });
        const parentBlockIndex = getBlockIndex({ blocks, id: blockToUnlink.parent });
        copiedBlocks[parentBlockIndex] = updateParentContentRef({
            parent: parentBlock,
            ids: [blockToUnlink.id],
            action: 'del',
        });
    }
    return copiedBlocks;
}
// function iterate(obj) {
//   Object.keys(obj).forEach(key => {
//     console.log(`key: ${key}, value: ${obj[key]}`)
//     if (typeof obj[key] === 'object' && obj[key] !== null) {
//       iterate(obj[key])
//     }
//   })
// }
export function getBlock({ blocks, id }) {
    return blocks.find(block => block.id === id);
}
function getBlockIndex({ blocks, id }) {
    return blocks.findIndex(block => block.id === id);
}
function getBlockAndIndex({ blocks, id }) {
    return { block: getBlock({ blocks, id }), index: getBlockIndex({ blocks, id }) };
}
function copyBlock(block) {
    console.log('copyBlock', block);
    const ts = new Date();
    return {
        ...block,
        id: uuid(),
        tc: ts,
        tm: ts,
        copyof: block.id,
    };
}
function copyBlockById({ blocks, id }) {
    return copyBlock(getBlock({ blocks, id }));
}
function modified(block) {
    return { ...block, tm: new Date() };
}
function updateParentContentRef({ parent, ids, action, }) {
    if (action === 'add')
        return modified({ ...parent, content: [...parent.content, ...ids] });
    else if (action === 'del')
        return modified({ ...parent, content: parent.content.filter(id => !ids.includes(id)) });
    else
        return parent;
}
/**
 * Copy a block and update the parent block's content array to reference the new block
 * @param blocks {Block[]} - The blocks array to update
 * @param id {Id} - The id of the block to copy
 * @returns {object} `{blocks, id (of copied block), index (of copied block)}`
 */
function copyAndUpdateParent({ blocks, id }) {
    const sourceBlockCopy = copyBlockById({ blocks: blocks, id });
    const copiedBlocks = [...blocks];
    if (sourceBlockCopy.parent) {
        const parentBlockIndex = getBlockIndex({ blocks: copiedBlocks, id: sourceBlockCopy.parent });
        const parentBlock = getBlock({ blocks: copiedBlocks, id: sourceBlockCopy.parent });
        copiedBlocks[parentBlockIndex] = updateParentContentRef({
            parent: parentBlock,
            ids: [sourceBlockCopy.id],
            action: 'add',
        });
    }
    copiedBlocks.push(sourceBlockCopy);
    const sourceBlockCopyIndex = getBlockIndex({ blocks: copiedBlocks, id: sourceBlockCopy.id });
    return {
        blocks: copiedBlocks,
        id: sourceBlockCopy.id,
        index: sourceBlockCopyIndex,
    };
}
function copyBlockContent({ blocks, id }) {
    let updatedBlocks = [...blocks];
    const sourceBlock = getBlock({ blocks, id });
    const sourceBlockIndex = getBlockIndex({ blocks, id });
    const sourceBlockContent = [...sourceBlock.content];
    updatedBlocks[sourceBlockIndex].content = [];
    // console.log('sourceBlockContent', sourceBlockContent)
    sourceBlockContent?.forEach(id => {
        const contentCopy = { ...copyBlockById({ blocks, id }), parent: sourceBlock.id };
        updatedBlocks[sourceBlockIndex].content = [...blocks[sourceBlockIndex].content, contentCopy.id];
        updatedBlocks = [...updatedBlocks, contentCopy];
        if (contentCopy.content.length) {
            copyBlockContent({ blocks: updatedBlocks, id: contentCopy.id });
        }
    });
    return updatedBlocks;
}
/**
 * Copy a block and update the parent block's content array to reference the new block. This does NOT copy the block's children.
 * @param {Block[]} blocks - The blocks array to update
 * @param {Id} id - The id of the block to copy
 */
export function copySelfOnly({ blocks, id }) {
    const updated = copyAndUpdateParent({ blocks, id });
    updated.blocks[updated.index].content = [];
    // console.log('updated.blocks', updated.blocks)
    // updatedBlocks[indexOfCopy].content = []
    return updated.blocks;
}
/**
 * Copy a block and all of its children recursively. This creates copies of all blocks in the tree with NEW unique ids.
 * @param blocks {Block[]} - The blocks array to update
 * @param id {Id} - The id of the block to copy
 */
export function copyRecursivelyAndUpdate({ blocks, id }) {
    const updated = copyAndUpdateParent({ blocks, id });
    return copyBlockContent({ blocks: updated.blocks, id: updated.id });
}
export function deleteContent({ blocks, id }) {
    // let updatedBlocks = [...blocks]
    const copyOfBlocks = [...blocks];
    const block = getBlock({ blocks, id });
    const blockIndex = getBlockIndex({ blocks: copyOfBlocks, id });
    const contentIdsToDelete = [];
    function getContendIdsRecursively(contentIds) {
        contentIds.forEach(id => {
            contentIdsToDelete.push(id);
            const childContentIds = getBlock({ blocks: copyOfBlocks, id })?.content;
            if (childContentIds) {
                getContendIdsRecursively(childContentIds);
            }
        });
    }
    getContendIdsRecursively(block.content);
    copyOfBlocks[blockIndex].content = [];
    return [...blocks].filter(block => !contentIdsToDelete.includes(block.id));
}
export function deleteBlockAndContent({ blocks, id }) {
    return removeRefToChild({ blocks: deleteContent({ blocks, id }), id }).filter(block => block.id !== id);
}
const blockPropDefaults = {
    text: {
        text: 'Text Block',
        tag: 'p',
    },
    timer: {
        text: 'Timer Block',
        start: new Date(),
        end: new Date(Date.now() + 60000),
    },
};
function createNewBlock(type) {
    const ts = new Date();
    return {
        id: uuid(),
        type,
        properties: blockPropDefaults[type],
        parent: null,
        content: [],
        tc: ts,
        tm: ts,
    };
}
export function addBlock({ blocks, type }) {
    return [...blocks, createNewBlock(type)];
}
